//
// Created by philipp on 08.05.18.
//

#include "../include/MessageLib/Client.h"
#include <iostream>

namespace message
{

Client::Client(Client::PrivateTag, networking::Networking & net, std::uint16_t port)
    : net(net), port(port)
{
    client = ServiceClient::create(net, MAX_MESSAGE_SIZE);
}

void Client::requestMessages(const std::string & host,
                             const networking::time::Duration & timeout,
                             const Client::OnMessageProcessedCallback & onMessageProcessedCallback,
                             const OnRequestFinishedCallback & onRequestFinishedCallback)
{
    using namespace std::chrono_literals;
    auto self = shared_from_this();
    auto state = std::make_shared<AsyncState>(self, onMessageProcessedCallback, onRequestFinishedCallback);
    std::string emptyRequest;
    client->asyncCall(
        emptyRequest, host, port, timeout,
        [state](const auto & error, auto & responseMessage)
        {
            if (error)
            {
                state->onRequestFinishedCallback(error);
                return;
            }

            if (!responseMessage.empty())
            {
                auto & parser = state->self->parser;
                parser << responseMessage;
                Message message;
                while (parser >> message)
                    state->onMessageProcessedCallback(message);
            }

            state->onRequestFinishedCallback(error);
        });
}

void Client::stop()
{
    client->stop();
}

}

